{ Directory rename for FreeDOS }

program drename;

// Version 1.1, (c) allu, January 2025

{$mode objfpc}{$H+}

uses
  SysUtils, Dos;

var
  P: Integer;
  Os, Source, Dest, Mess: string;
  Keep: Boolean;
  OutFile: Text;

{ Display program usage }

procedure DisplayUsage;

begin
  Writeln('Usage');
  Writeln('  drename /H');
  Writeln('  drename [/K] [drive:\][path\]directory1 directory2');
end;

{ Display program help }

procedure DisplayHelp;

begin
  DisplayUsage;
  Writeln('Use /K to keep directory1 (duplicate instead of rename).');
  Writeln('Note that directory2 must not include a drive or a path.');
end;

{ ************ }
{ Main program }
{ ************ }

begin
  Os := GetEnv('OS_NAME');
  if Os = 'FreeDOS' then begin
    // Run only, if we are on FreeDOS
    Keep := False; Mess := '';
    if (ParamCount = 1) and ((ParamStr(1) = '/H') or (ParamStr(1) = '/h')) then begin
      // drename /H: Display program help
      DisplayHelp;
    end
    else if (ParamCount = 2) or (ParamCount = 3) then begin
      // General case (drename [/K] source destination): Rename or duplicate
      if (ParamCount = 3) then begin
        // If there are 3 command line arguments, the first one must be /K
        // The second and third one are the source and destination
        if (ParamStr(1) = '/K') or (ParamStr(1) = '/k') then begin
          Source := ParamStr(2); Dest := ParamStr(3);
          Keep := True;                                                                  // /K: keep source direcctory (-> directory duplication instead of rename)
        end
        else begin
          Mess := 'Invalid parameter: ' + ParamStr(1);
        end;
      end
      else begin
        // If there are two command line arguments, they are the source and destination
        if (ParamStr(1) = '/k') or (ParamStr(1) = '/K') then
          Mess := 'Wrong number of parameters'
        else begin
          Source := ParamStr(1); Dest := ParamStr(2);
        end;
      end;
      if Mess = '' then begin
        // Command line arguments are all there; check directory names
        if not DirectoryExists(Source) then
          // The source directory must exist
          Mess := 'Directory not found: ' + Source
        else begin
          // The destination directory must not include a drive letter or path
          P := Pos(':', Dest) + Pos('\', Dest);
          if P <> 0 then
            Mess := 'Invalid directory syntax: ' + Dest;
        end;
        if Mess = '' then begin
          // The destination directory must not exist
          Dest := ExtractFilePath(Source) + Dest;
          if DirectoryExists(Dest) then
            Mess := 'Directory already exists: ' + Dest;
        end;
      end;
      if Mess = '' then begin
        // Command line arguments are all correct; create the batch file and run it
        Assign(OutFile, 'drename.bat'); Rewrite(OutFile);
        Writeln(OutFile, '@echo off');
        Writeln(OutFile, 'mkdir ' + Dest);                                               // create destination directory
        Writeln(OutFile, 'if not errorlevel 0 goto End');
        Writeln(OutFile, 'xcopy ' + Source + '\*.* ' + Dest + ' /E /I /H /Q');           // copy all files and subdirectories from the source to the destination
        if not Keep then
          Writeln(OutFile, 'deltree /Y ' + Source);                                      // delete the source directory tree (unless /K is specified)
        Writeln(OutFile, ':End');
        Close(OutFile);
        // Run the batch file
        Exec(GetEnv('COMSPEC'), '/C drename.bat');
        // DOSError should always be 0
        if DosError <> 0 then
          Mess := 'DOS error: Error code = ' + IntToStr(DosError);
      end;
    end
    else begin
      Mess := 'Wrong number of parameters';
    end;
  end
  else
    Mess := 'Sorry, this is a FreeDOS program...';
  if FileExists('drename.bat') then
    DeleteFile('drename.bat');
  if Mess <> '' then begin
    // If there has been an error, display message
    Writeln(Mess, '!');
    if Copy(Mess, 1, 3) <> 'DOS' then
      DisplayUsage;                                                                      // if user error, display program usage
  end;
end.
